import { Event } from "@codingame/monaco-vscode-api/vscode/vs/base/common/event";
import { IMarkdownString } from "@codingame/monaco-vscode-api/vscode/vs/base/common/htmlContent";
import { IDisposable, IReference } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { RawContextKey } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey";
import { IRevertOptions, ISaveOptions } from "@codingame/monaco-vscode-api/vscode/vs/workbench/common/editor";
import { RegisteredEditorPriority } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorResolverService";
export declare const CONTEXT_ACTIVE_CUSTOM_EDITOR_ID: RawContextKey<string>;
export declare const CONTEXT_FOCUSED_CUSTOM_EDITOR_IS_EDITABLE: RawContextKey<boolean>;
export interface CustomEditorCapabilities {
    readonly supportsMultipleEditorsPerDocument?: boolean;
}
export interface ICustomEditorModelManager {
    getAllModels(resource: URI): Promise<ICustomEditorModel[]>;
    get(resource: URI, viewType: string): Promise<ICustomEditorModel | undefined>;
    tryRetain(resource: URI, viewType: string): Promise<IReference<ICustomEditorModel>> | undefined;
    add(resource: URI, viewType: string, model: Promise<ICustomEditorModel>): Promise<IReference<ICustomEditorModel>>;
    disposeAllModelsForView(viewType: string): void;
}
export interface ICustomEditorModel extends IDisposable {
    readonly viewType: string;
    readonly resource: URI;
    readonly backupId: string | undefined;
    readonly canHotExit: boolean;
    isReadonly(): boolean | IMarkdownString;
    readonly onDidChangeReadonly: Event<void>;
    isOrphaned(): boolean;
    readonly onDidChangeOrphaned: Event<void>;
    isDirty(): boolean;
    readonly onDidChangeDirty: Event<void>;
    revert(options?: IRevertOptions): Promise<void>;
    saveCustomEditor(options?: ISaveOptions): Promise<URI | undefined>;
    saveCustomEditorAs(resource: URI, targetResource: URI, currentOptions?: ISaveOptions): Promise<boolean>;
}
export declare enum CustomEditorPriority {
    default = "default",
    builtin = "builtin",
    option = "option"
}
export interface CustomEditorSelector {
    readonly filenamePattern?: string;
}
export interface CustomEditorDescriptor {
    readonly id: string;
    readonly displayName: string;
    readonly providerDisplayName: string;
    readonly priority: RegisteredEditorPriority;
    readonly selector: readonly CustomEditorSelector[];
}
export declare class CustomEditorInfo implements CustomEditorDescriptor {
    readonly id: string;
    readonly displayName: string;
    readonly providerDisplayName: string;
    readonly priority: RegisteredEditorPriority;
    readonly selector: readonly CustomEditorSelector[];
    constructor(descriptor: CustomEditorDescriptor);
    matches(resource: URI): boolean;
}
export declare class CustomEditorInfoCollection {
    readonly allEditors: readonly CustomEditorInfo[];
    constructor(editors: readonly CustomEditorInfo[]);
    get length(): number;
    get defaultEditor(): CustomEditorInfo | undefined;
    get bestAvailableEditor(): CustomEditorInfo | undefined;
}
