import { IMouseWheelEvent } from "@codingame/monaco-vscode-api/vscode/vs/base/browser/mouseEvent";
import { CodeWindow } from "@codingame/monaco-vscode-api/vscode/vs/base/browser/window";
import { VSBufferReadableStream } from "@codingame/monaco-vscode-api/vscode/vs/base/common/buffer";
import { Emitter, Event } from "@codingame/monaco-vscode-api/vscode/vs/base/common/event";
import { Disposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { IAccessibilityService } from "@codingame/monaco-vscode-api/vscode/vs/platform/accessibility/common/accessibility.service";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
import { IContextMenuService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextview/browser/contextView.service";
import { ExtensionIdentifier } from "@codingame/monaco-vscode-api/vscode/vs/platform/extensions/common/extensions";
import { IFileService } from "@codingame/monaco-vscode-api/vscode/vs/platform/files/common/files.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { INotificationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification.service";
import { IRemoteAuthorityResolverService } from "@codingame/monaco-vscode-api/vscode/vs/platform/remote/common/remoteAuthorityResolver.service";
import { ITunnelService } from "@codingame/monaco-vscode-api/vscode/vs/platform/tunnel/common/tunnel.service";
import { IWorkbenchEnvironmentService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/environment/common/environmentService.service";
import { WebviewThemeDataProvider } from "./themeing.js";
import { IWebview, WebviewContentOptions, WebviewExtensionDescription, WebviewInitInfo, WebviewMessageReceivedEvent } from "@codingame/monaco-vscode-7ba0af96-90c2-5e11-ad7f-befdbbf246c8-common/vscode/vs/workbench/contrib/webview/browser/webview";
import { WebviewFindDelegate, WebviewFindWidget } from "./webviewFindWidget.js";
export declare class WebviewElement extends Disposable implements IWebview, WebviewFindDelegate {
    protected readonly webviewThemeDataProvider: WebviewThemeDataProvider;
    private readonly _environmentService;
    private readonly _fileService;
    private readonly _logService;
    private readonly _remoteAuthorityResolverService;
    private readonly _tunnelService;
    private readonly _accessibilityService;
    protected readonly id: string;
    readonly providedViewType?: string;
    readonly origin: string;
    private _windowId;
    private get window();
    private _encodedWebviewOriginPromise?;
    private _encodedWebviewOrigin;
    protected get platform(): string;
    private readonly _expectedServiceWorkerVersion;
    private _element;
    protected get element(): HTMLIFrameElement | undefined;
    private _focused;
    get isFocused(): boolean;
    private _state;
    private _content;
    private readonly _portMappingManager;
    private readonly _resourceLoadingCts;
    private _contextKeyService;
    private _confirmBeforeClose;
    private readonly _focusDelayer;
    private readonly _onDidHtmlChange;
    protected readonly onDidHtmlChange: Event<string>;
    private _messagePort?;
    private readonly _messageHandlers;
    protected readonly _webviewFindWidget: WebviewFindWidget | undefined;
    readonly checkImeCompletionState = true;
    private _disposed;
    extension: WebviewExtensionDescription | undefined;
    private readonly _options;
    constructor(initInfo: WebviewInitInfo, webviewThemeDataProvider: WebviewThemeDataProvider, configurationService: IConfigurationService, contextMenuService: IContextMenuService, notificationService: INotificationService, _environmentService: IWorkbenchEnvironmentService, _fileService: IFileService, _logService: ILogService, _remoteAuthorityResolverService: IRemoteAuthorityResolverService, _tunnelService: ITunnelService, instantiationService: IInstantiationService, _accessibilityService: IAccessibilityService);
    dispose(): void;
    setContextKeyService(contextKeyService: IContextKeyService): void;
    private readonly _onMissingCsp;
    readonly onMissingCsp: Event<ExtensionIdentifier>;
    private readonly _onDidClickLink;
    readonly onDidClickLink: Event<string>;
    private readonly _onDidReload;
    readonly onDidReload: Event<void>;
    private readonly _onMessage;
    readonly onMessage: Event<WebviewMessageReceivedEvent>;
    private readonly _onDidScroll;
    readonly onDidScroll: Event<{
        readonly scrollYPercentage: number;
    }>;
    private readonly _onDidWheel;
    readonly onDidWheel: Event<IMouseWheelEvent>;
    private readonly _onDidUpdateState;
    readonly onDidUpdateState: Event<string | undefined>;
    private readonly _onDidFocus;
    readonly onDidFocus: Event<void>;
    private readonly _onDidBlur;
    readonly onDidBlur: Event<void>;
    private readonly _onFatalError;
    readonly onFatalError: Event<{
        readonly message: string;
    }>;
    private readonly _onDidDispose;
    readonly onDidDispose: Event<void>;
    postMessage(message: any, transfer?: ArrayBuffer[]): Promise<boolean>;
    private _send;
    private _createElement;
    private _initElement;
    mountTo(element: HTMLElement, targetWindow: CodeWindow): void;
    private _registerMessageHandler;
    private _startBlockingIframeDragEvents;
    private _stopBlockingIframeDragEvents;
    protected webviewContentEndpoint(encodedWebviewOrigin: string): string;
    private doPostMessage;
    private on;
    private _hasAlertedAboutMissingCsp;
    private handleNoCspFound;
    reload(): void;
    setHtml(html: string): void;
    setTitle(title: string): void;
    set contentOptions(options: WebviewContentOptions);
    set localResourcesRoot(resources: readonly URI[]);
    set state(state: string | undefined);
    set initialScrollProgress(value: number);
    private doUpdateContent;
    protected style(): void;
    protected handleFocusChange(isFocused: boolean): void;
    private handleKeyEvent;
    private handleDragEvent;
    windowDidDragStart(): void;
    windowDidDragEnd(): void;
    selectAll(): void;
    copy(): void;
    paste(): void;
    cut(): void;
    undo(): void;
    redo(): void;
    private execCommand;
    private loadResource;
    protected streamToBuffer(stream: VSBufferReadableStream): Promise<ArrayBufferLike>;
    private localLocalhost;
    focus(): void;
    private _doFocus;
    protected readonly _hasFindResult: Emitter<boolean>;
    readonly hasFindResult: Event<boolean>;
    protected readonly _onDidStopFind: Emitter<void>;
    readonly onDidStopFind: Event<void>;
    find(value: string, previous: boolean): void;
    updateFind(value: string): void;
    stopFind(keepSelection?: boolean): void;
    showFind(animated?: boolean): void;
    hideFind(animated?: boolean): void;
    runFindAction(previous: boolean): void;
}
