import * as glob from "@codingame/monaco-vscode-api/vscode/vs/base/common/glob";
import { Disposable, IDisposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IUntypedEditorInput } from "@codingame/monaco-vscode-api/vscode/vs/workbench/common/editor";
import { IEditorGroupsService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorGroupsService.service";
import { RegisteredEditorInfo, RegisteredEditorOptions, EditorAssociations, ResolvedEditor, EditorInputFactoryObject } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorResolverService";
import { IEditorResolverService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorResolverService.service";
import { IQuickInputService } from "@codingame/monaco-vscode-api/vscode/vs/platform/quickinput/common/quickInput.service";
import { INotificationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification.service";
import { IStorageService } from "@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage.service";
import { IExtensionService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/extensions/common/extensions.service";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { PreferredGroup } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorService";
export declare class EditorResolverService extends Disposable implements IEditorResolverService {
    private readonly editorGroupService;
    private readonly instantiationService;
    private readonly configurationService;
    private readonly quickInputService;
    private readonly notificationService;
    private readonly storageService;
    private readonly extensionService;
    private readonly logService;
    readonly _serviceBrand: undefined;
    private readonly _onDidChangeEditorRegistrations;
    readonly onDidChangeEditorRegistrations: import("@codingame/monaco-vscode-api/vscode/vs/base/common/event").Event<void>;
    private static readonly configureDefaultID;
    private static readonly cacheStorageID;
    private static readonly conflictingDefaultsStorageID;
    private _editors;
    private _flattenedEditors;
    private _shouldReFlattenEditors;
    private cache;
    constructor(editorGroupService: IEditorGroupsService, instantiationService: IInstantiationService, configurationService: IConfigurationService, quickInputService: IQuickInputService, notificationService: INotificationService, storageService: IStorageService, extensionService: IExtensionService, logService: ILogService);
    private resolveUntypedInputAndGroup;
    resolveEditor(editor: IUntypedEditorInput, preferredGroup: PreferredGroup | undefined): Promise<ResolvedEditor>;
    private doResolveSideBySideEditor;
    bufferChangeEvents(callback: Function): void;
    registerEditor(globPattern: string | glob.IRelativePattern, editorInfo: RegisteredEditorInfo, options: RegisteredEditorOptions, editorFactoryObject: EditorInputFactoryObject): IDisposable;
    getAssociationsForResource(resource: URI): EditorAssociations;
    getAllUserAssociations(): EditorAssociations;
    private _flattenEditorsMap;
    private get _registeredEditors();
    updateUserAssociations(globPattern: string, editorID: string): void;
    private findMatchingEditors;
    getEditors(resource?: URI): RegisteredEditorInfo[];
    private getEditor;
    private doResolveEditor;
    private moveExistingEditorForResource;
    private findExistingEditorsForResource;
    private doHandleConflictingDefaults;
    private mapEditorsToQuickPickEntry;
    private doPickEditor;
    private cacheEditors;
    private resourceMatchesCache;
}
